@@ -29,7 +29,7 @@
  * implementation of fonts. The #PangoFont structure
  * represents an abstract rendering-system-independent font.
  * Pango provides routines to list available fonts, and
- * to load a font of a given description.
+ * to load a font matching a given description.
  */
 
 #include "config.h"
@@ -38,7 +38,7 @@
 #include <string.h>
 
 #include "pango-types.h"
-#include "pango-font.h"
+#include "pango-font-private.h"
 #include "pango-fontmap.h"
 #include "pango-impl-utils.h"
 
@@ -52,8 +52,11 @@ struct _PangoFontDescription
   PangoStretch stretch;
   PangoGravity gravity;
 
+  char *variations;
+
   guint16 mask;
   guint static_family : 1;
+  guint static_variations : 1;
   guint size_is_absolute : 1;
 
   int size;
@@ -71,10 +74,12 @@ static const PangoFontDescription pfd_defaults = {
   PANGO_WEIGHT_NORMAL,	/* weight */
   PANGO_STRETCH_NORMAL,	/* stretch */
   PANGO_GRAVITY_SOUTH,  /* gravity */
+  NULL,                 /* variations */
 
   0,			/* mask */
   0,			/* static_family */
-  FALSE,		/* size_is_absolute */
+  0,			/* static_variations*/
+  0,    		/* size_is_absolute */
 
   0,			/* size */
 };
@@ -164,9 +169,10 @@ pango_font_description_set_family_static (PangoFontDescription *desc,
  * Gets the family name field of a font description. See
  * pango_font_description_set_family().
  *
- * Return value: the family name field for the font description, or
- *               %NULL if not previously set.  This has the same life-time
- *               as the font description itself and should not be freed.
+ * Return value: (nullable): the family name field for the font
+ *               description, or %NULL if not previously set.  This
+ *               has the same life-time as the font description itself
+ *               and should not be freed.
  **/
 const char *
 pango_font_description_get_family (const PangoFontDescription *desc)
@@ -478,6 +484,98 @@ pango_font_description_get_gravity (const PangoFontDescription *desc)
   return desc->gravity;
 }
 
+/**
+ * pango_font_description_set_variations_static:
+ * @desc: a #PangoFontDescription
+ * @variations: a string representing the variations
+ *
+ * Like pango_font_description_set_variations(), except that no
+ * copy of @variations is made. The caller must make sure that the
+ * string passed in stays around until @desc has been freed
+ * or the name is set again. This function can be used if
+ * @variations is a static string such as a C string literal, or
+ * if @desc is only needed temporarily.
+ *
+ * Since: 1.42
+ **/
+void
+pango_font_description_set_variations_static (PangoFontDescription *desc,
+                                              const char           *variations)
+{
+  g_return_if_fail (desc != NULL);
+
+  if (desc->variations == variations)
+    return;
+
+  if (desc->variations && !desc->static_variations)
+    g_free (desc->variations);
+
+  if (variations)
+    {
+      desc->variations = (char *)variations;
+      desc->static_variations = TRUE;
+      desc->mask |= PANGO_FONT_MASK_VARIATIONS;
+    }
+  else
+    {
+      desc->variations = pfd_defaults.variations;
+      desc->static_variations = pfd_defaults.static_variations;
+      desc->mask &= ~PANGO_FONT_MASK_VARIATIONS;
+    }
+}
+
+/**
+ * pango_font_description_set_variations:
+ * @desc: a #PangoFontDescription.
+ * @variations: a string representing the variations
+ *
+ * Sets the variations field of a font description. OpenType
+ * font variations allow to select a font instance by specifying
+ * values for a number of axes, such as width or weight.
+ *
+ * The format of the variations string is AXIS1=VALUE,AXIS2=VALUE...,
+ * with each AXIS a 4 character tag that identifies a font axis,
+ * and each VALUE a floating point number. Unknown axes are ignored,
+ * and values are clamped to their allowed range.
+ *
+ * Pango does not currently have a way to find supported axes of
+ * a font. Both harfbuzz or freetype have API for this.
+ *
+ * Since: 1.42
+ **/
+void
+pango_font_description_set_variations (PangoFontDescription *desc,
+                                       const char           *variations)
+{
+  g_return_if_fail (desc != NULL);
+
+  pango_font_description_set_variations_static (desc, g_strdup (variations));
+  if (variations)
+    desc->static_variations = FALSE;
+}
+
+/**
+ * pango_font_description_get_variations:
+ * @desc: a #PangoFontDescription
+ *
+ * Gets the variations field of a font description. See
+ * pango_font_description_set_variations().
+ *
+ * Return value: (nullable): the varitions field for the font
+ *               description, or %NULL if not previously set.  This
+ *               has the same life-time as the font description itself
+ *               and should not be freed.
+ *
+ * Since: 1.42
+ **/
+const char *
+pango_font_description_get_variations (const PangoFontDescription *desc)
+{
+  g_return_val_if_fail (desc != NULL, NULL);
+
+  return desc->variations;
+}
+
 /**
  * pango_font_description_get_set_fields:
  * @desc: a #PangoFontDescription
@@ -540,6 +638,7 @@ pango_font_description_merge (PangoFontDescription       *desc,
 			      gboolean                    replace_existing)
 {
   gboolean family_merged;
+  gboolean variations_merged;
 
   g_return_if_fail (desc != NULL);
 
@@ -547,6 +646,7 @@ pango_font_description_merge (PangoFontDescription       *desc,
     return;
 
   family_merged = desc_to_merge->family_name && (replace_existing || !desc->family_name);
+  variations_merged = desc_to_merge->variations && (replace_existing || !desc->variations);
 
   pango_font_description_merge_static (desc, desc_to_merge, replace_existing);
 
@@ -555,6 +655,12 @@ pango_font_description_merge (PangoFontDescription       *desc,
       desc->family_name = g_strdup (desc->family_name);
       desc->static_family = FALSE;
     }
+
+  if (variations_merged)
+    {
+      desc->variations = g_strdup (desc->variations);
+      desc->static_variations = FALSE;
+    }
 }
 
 /**
@@ -602,6 +708,8 @@ pango_font_description_merge_static (PangoFontDescription       *desc,
     }
   if (new_mask & PANGO_FONT_MASK_GRAVITY)
     desc->gravity = desc_to_merge->gravity;
+  if (new_mask & PANGO_FONT_MASK_VARIATIONS)
+    pango_font_description_set_variations_static (desc, desc_to_merge->variations);
 
   desc->mask |= new_mask;
 }
@@ -612,14 +720,14 @@ compute_distance (const PangoFontDescription *a,
 {
   if (a->style == b->style)
     {
-      return abs(a->weight - b->weight);
+      return abs((int)(a->weight) - (int)(b->weight));
     }
   else if (a->style != PANGO_STYLE_NORMAL &&
 	   b->style != PANGO_STYLE_NORMAL)
     {
       /* Equate oblique and italic, but with a big penalty
        */
-      return 1000000 + abs (a->weight - b->weight);
+      return 1000000 + abs ((int)(a->weight) - (int)(b->weight));
     }
   else
     return G_MAXINT;
@@ -669,13 +777,14 @@ pango_font_description_better_match (const PangoFontDescription *desc,
 
 /**
  * pango_font_description_copy:
- * @desc: a #PangoFontDescription, may be %NULL
+ * @desc: (nullable): a #PangoFontDescription, may be %NULL
  *
  * Make a copy of a #PangoFontDescription.
  *
- * Return value: the newly allocated #PangoFontDescription, which should
- *               be freed with pango_font_description_free(), or %NULL
- *               if @desc was %NULL.
+ * Return value: (nullable): the newly allocated
+ *               #PangoFontDescription, which should be freed with
+ *               pango_font_description_free(), or %NULL if @desc was
+ *               %NULL.
  **/
 PangoFontDescription *
 pango_font_description_copy  (const PangoFontDescription  *desc)
@@ -695,21 +804,25 @@ pango_font_description_copy  (const PangoFontDescription  *desc)
       result->static_family = FALSE;
     }
 
+  result->variations = g_strdup (result->variations);
+  result->static_variations = FALSE;
+
   return result;
 }
 
 /**
  * pango_font_description_copy_static:
- * @desc: a #PangoFontDescription, may be %NULL
+ * @desc: (nullable): a #PangoFontDescription, may be %NULL
  *
  * Like pango_font_description_copy(), but only a shallow copy is made
  * of the family name and other allocated fields. The result can only
  * be used until @desc is modified or freed. This is meant to be used
  * when the copy is only needed temporarily.
  *
- * Return value: the newly allocated #PangoFontDescription, which should
- *               be freed with pango_font_description_free(), or %NULL
- *               if @desc was %NULL.
+ * Return value: (nullable): the newly allocated
+ *               #PangoFontDescription, which should be freed with
+ *               pango_font_description_free(), or %NULL if @desc was
+ *               %NULL.
  **/
 PangoFontDescription *
 pango_font_description_copy_static (const PangoFontDescription *desc)
@@ -725,6 +838,10 @@ pango_font_description_copy_static (const PangoFontDescription *desc)
   if (result->family_name)
     result->static_family = TRUE;
 
+
+  if (result->variations)
+    result->static_variations = TRUE;
+
   return result;
 }
 
@@ -757,7 +874,8 @@ pango_font_description_equal (const PangoFontDescription  *desc1,
 	 desc1->size_is_absolute == desc2->size_is_absolute &&
 	 desc1->gravity == desc2->gravity &&
 	 (desc1->family_name == desc2->family_name ||
-	  (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0));
+	  (desc1->family_name && desc2->family_name && g_ascii_strcasecmp (desc1->family_name, desc2->family_name) == 0)) &&
+         (g_strcmp0 (desc1->variations, desc2->variations) == 0);
 }
 
 #define TOLOWER(c) \
@@ -797,6 +915,8 @@ pango_font_description_hash (const PangoFontDescription *desc)
 
   if (desc->family_name)
     hash = case_insensitive_hash (desc->family_name);
+  if (desc->variations)
+    hash ^= g_str_hash (desc->variations);
   hash ^= desc->size;
   hash ^= desc->size_is_absolute ? 0xc33ca55a : 0;
   hash ^= desc->style << 16;
@@ -810,7 +930,7 @@ pango_font_description_hash (const PangoFontDescription *desc)
 
 /**
  * pango_font_description_free:
- * @desc: a #PangoFontDescription, may be %NULL
+ * @desc: (nullable): a #PangoFontDescription, may be %NULL
  *
  * Frees a font description.
  **/
@@ -823,6 +943,9 @@ pango_font_description_free  (PangoFontDescription  *desc)
   if (desc->family_name && !desc->static_family)
     g_free (desc->family_name);
 
+  if (desc->variations && !desc->static_variations)
+    g_free (desc->variations);
+
   g_slice_free (PangoFontDescription, desc);
 }
 
@@ -871,6 +994,8 @@ static const FieldMap weight_map[] = {
   { PANGO_WEIGHT_ULTRALIGHT, "Ultra-Light" },
   { PANGO_WEIGHT_ULTRALIGHT, "Extra-Light" },
   { PANGO_WEIGHT_LIGHT, "Light" },
+  { PANGO_WEIGHT_SEMILIGHT, "Semi-Light" },
+  { PANGO_WEIGHT_SEMILIGHT, "Demi-Light" },
   { PANGO_WEIGHT_BOOK, "Book" },
   { PANGO_WEIGHT_NORMAL, "" },
   { PANGO_WEIGHT_NORMAL, "Regular" },
@@ -1026,7 +1151,7 @@ find_field_any (const char *str, int len, PangoFontDescription *desc)
 }
 
 static const char *
-getword (const char *str, const char *last, size_t *wordlen)
+getword (const char *str, const char *last, size_t *wordlen, const char *stop)
 {
   const char *result;
 
@@ -1034,7 +1159,7 @@ getword (const char *str, const char *last, size_t *wordlen)
     last--;
 
   result = last;
-  while (result > str && !g_ascii_isspace (*(result - 1)) && *(result - 1) != ',')
+  while (result > str && !g_ascii_isspace (*(result - 1)) && !strchr (stop, *(result - 1)))
     result--;
 
   *wordlen = last - result;
@@ -1068,22 +1193,69 @@ parse_size (const char *word,
   return FALSE;
 }
 
+static gboolean
+parse_variations (const char  *word,
+                  size_t       wordlen,
+                  char       **variations)
+{
+  if (word[0] != '@')
+    {
+      *variations = NULL;
+      return FALSE;
+    }
+
+  /* XXX: actually validate here */
+  *variations = g_strndup (word + 1, wordlen - 1);
+
+  return TRUE;
+}
+
 /**
  * pango_font_description_from_string:
  * @str: string representation of a font description.
  *
  * Creates a new font description from a string representation in the
- * form "[FAMILY-LIST] [STYLE-OPTIONS] [SIZE]", where FAMILY-LIST is a
- * comma separated list of families optionally terminated by a comma,
- * STYLE_OPTIONS is a whitespace separated list of words where each WORD
- * describes one of style, variant, weight, stretch, or gravity, and SIZE
- * is a decimal number (size in points) or optionally followed by the
- * unit modifier "px" for absolute size. Any one of the options may
- * be absent.  If FAMILY-LIST is absent, then the family_name field of
- * the resulting font description will be initialized to %NULL.  If
- * STYLE-OPTIONS is missing, then all style options will be set to the
- * default values. If SIZE is missing, the size in the resulting font
- * description will be set to 0.
+ * form
+ *
+ * "\[FAMILY-LIST] \[STYLE-OPTIONS] \[SIZE] \[VARIATIONS]",
+ *
+ * where FAMILY-LIST is a comma-separated list of families optionally
+ * terminated by a comma, STYLE_OPTIONS is a whitespace-separated list
+ * of words where each word describes one of style, variant, weight,
+ * stretch, or gravity, and SIZE is a decimal number (size in points)
+ * or optionally followed by the unit modifier "px" for absolute size.
+ * VARIATIONS is a comma-separated list of font variation
+ * specifications of the form "\@axis=value" (the = sign is optional).
+ *
+ * The following words are understood as styles:
+ * "Normal", "Roman", "Oblique", "Italic".
+ *
+ * The following words are understood as variants:
+ * "Small-Caps".
+ *
+ * The following words are understood as weights:
+ * "Thin", "Ultra-Light", "Extra-Light", "Light", "Semi-Light",
+ * "Demi-Light", "Book", "Regular", "Medium", "Semi-Bold", "Demi-Bold",
+ * "Bold", "Ultra-Bold", "Extra-Bold", "Heavy", "Black", "Ultra-Black",
+ * "Extra-Black".
+ *
+ * The following words are understood as stretch values:
+ * "Ultra-Condensed", "Extra-Condensed", "Condensed", "Semi-Condensed",
+ * "Semi-Expanded", "Expanded", "Extra-Expanded", "Ultra-Expanded".
+ *
+ * The following words are understood as gravity values:
+ * "Not-Rotated", "South", "Upside-Down", "North", "Rotated-Left",
+ * "East", "Rotated-Right", "West".
+ *
+ * Any one of the options may be absent. If FAMILY-LIST is absent, then
+ * the family_name field of the resulting font description will be
+ * initialized to %NULL. If STYLE-OPTIONS is missing, then all style
+ * options will be set to the default values. If SIZE is missing, the
+ * size in the resulting font description will be set to 0.
+ *
+ * A typical example:
+ *
+ * "Cantarell Italic Light 15 \@wght=200"
  *
  * Return value: a new #PangoFontDescription.
  **/
@@ -1105,10 +1277,19 @@ pango_font_description_from_string (const char *str)
 
   len = strlen (str);
   last = str + len;
-  p = getword (str, last, &wordlen);
+  p = getword (str, last, &wordlen, "");
+  /* Look for variations at the end of the string */
+  if (wordlen != 0)
+    {
+      if (parse_variations (p, wordlen, &desc->variations))
+        {
+	  desc->mask |= PANGO_FONT_MASK_VARIATIONS;
+	  last = p;
+        }
+    }
 
-  /* Look for a size at the end of the string
-   */
+  p = getword (str, last, &wordlen, ",");
+  /* Look for a size */
   if (wordlen != 0)
     {
       gboolean size_is_absolute;
@@ -1122,7 +1303,7 @@ pango_font_description_from_string (const char *str)
 
   /* Now parse style words
    */
-  p = getword (str, last, &wordlen);
+  p = getword (str, last, &wordlen, ",");
   while (wordlen != 0)
     {
       if (!find_field_any (p, wordlen, desc))
@@ -1130,7 +1311,7 @@ pango_font_description_from_string (const char *str)
       else
 	{
 	  last = p;
-	  p = getword (str, last, &wordlen);
+	  p = getword (str, last, &wordlen, ",");
 	}
     }
 
@@ -1183,7 +1364,7 @@ append_field (GString *str, const char *what, const FieldMap *map, int n_element
       if (map[i].value != val)
         continue;
 
-      if (G_LIKELY (map[i].str && map[i].str[0]))
+      if (G_LIKELY (map[i].str[0]))
 	{
 	  if (G_LIKELY (str->len > 0 && str->str[str->len -1] != ' '))
 	    g_string_append_c (str, ' ');
@@ -1229,7 +1410,7 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
        * in a keyword like "Bold", or if the family name ends in
        * a number and no keywords will be added.
        */
-      p = getword (desc->family_name, desc->family_name + strlen(desc->family_name), &wordlen);
+      p = getword (desc->family_name, desc->family_name + strlen(desc->family_name), &wordlen, ",");
       if (wordlen != 0 &&
 	  (find_field_any (p, wordlen, NULL) ||
 	   (parse_size (p, wordlen, NULL, NULL) &&
@@ -1270,6 +1451,12 @@ pango_font_description_to_string (const PangoFontDescription  *desc)
 	g_string_append (result, "px");
     }
 
+  if (desc->variations && desc->mask & PANGO_FONT_MASK_VARIATIONS)
+    {
+      g_string_append (result, " @");
+      g_string_append (result, desc->variations);
+    }
+
   return g_string_free (result, FALSE);
 }
 
@@ -1370,7 +1557,7 @@ parse_field (const char *what,
 /**
  * pango_parse_style:
  * @str: a string to parse.
- * @style: (out caller-allocates): a #PangoStyle to store the result
+ * @style: (out): a #PangoStyle to store the result
  *   in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  *
@@ -1391,7 +1578,7 @@ pango_parse_style (const char *str,
 /**
  * pango_parse_variant:
  * @str: a string to parse.
- * @variant: (out caller-allocates): a #PangoVariant to store the
+ * @variant: (out): a #PangoVariant to store the
  *   result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  *
@@ -1412,7 +1599,7 @@ pango_parse_variant (const char   *str,
 /**
  * pango_parse_weight:
  * @str: a string to parse.
- * @weight: (out caller-allocates): a #PangoWeight to store the result
+ * @weight: (out): a #PangoWeight to store the result
  *   in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  *
@@ -1433,7 +1620,7 @@ pango_parse_weight (const char  *str,
 /**
  * pango_parse_stretch:
  * @str: a string to parse.
- * @stretch: (out caller-allocates): a #PangoStretch to store the
+ * @stretch: (out): a #PangoStretch to store the
  *   result in.
  * @warn: if %TRUE, issue a g_warning() on bad input.
  *
@@ -1460,11 +1647,29 @@ pango_parse_stretch (const char   *str,
  * PangoFont
  */
 
-G_DEFINE_ABSTRACT_TYPE (PangoFont, pango_font, G_TYPE_OBJECT)
+typedef struct {
+  hb_font_t *hb_font;
+} PangoFontPrivate;
+
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (PangoFont, pango_font, G_TYPE_OBJECT)
+
+static void
+pango_font_finalize (GObject *object)
+{
+  PangoFont *font = PANGO_FONT (object);
+  PangoFontPrivate *priv = pango_font_get_instance_private (font);
+
+  hb_font_destroy (priv->hb_font);
+
+  G_OBJECT_CLASS (pango_font_parent_class)->finalize (object);
+}
 
 static void
 pango_font_class_init (PangoFontClass *class G_GNUC_UNUSED)
 {
+  GObjectClass *object_class = G_OBJECT_CLASS (class);
+
+  object_class->finalize = pango_font_finalize;
 }
 
 static void
@@ -1544,25 +1749,20 @@ pango_font_get_coverage (PangoFont     *font,
  * Finds the best matching shaper for a font for a particular
  * language tag and character point.
  *
- * Return value: the best matching shaper.
+ * Return value: (transfer none): the best matching shaper.
+ * Deprecated: Shape engines are no longer used
  **/
 PangoEngineShape *
 pango_font_find_shaper (PangoFont     *font,
 			PangoLanguage *language,
 			guint32        ch)
 {
-  PangoEngineShape* shaper;
-
-  g_return_val_if_fail (font != NULL, NULL);
-
-  shaper = PANGO_FONT_GET_CLASS (font)->find_shaper (font, language, ch);
-
-  return shaper;
+  return NULL;
 }
 
 /**
  * pango_font_get_glyph_extents:
- * @font: a #PangoFont
+ * @font: (nullable): a #PangoFont
  * @glyph: the glyph index
  * @ink_rect: (out) (allow-none): rectangle used to store the extents of the glyph
  *            as drawn or %NULL to indicate that the result is not needed.
@@ -1610,7 +1810,7 @@ pango_font_get_glyph_extents  (PangoFont      *font,
 
 /**
  * pango_font_get_metrics:
- * @font: a #PangoFont
+ * @font: (nullable): a #PangoFont
  * @language: (allow-none): language tag used to determine which script to get the metrics
  *            for, or %NULL to indicate to get the metrics for the entire font.
  *
@@ -1635,6 +1835,7 @@ pango_font_get_metrics (PangoFont        *font,
 
       metrics->ascent = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_HEIGHT;
       metrics->descent = 0;
+      metrics->height = 0;
       metrics->approximate_char_width = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_WIDTH;
       metrics->approximate_digit_width = PANGO_SCALE * PANGO_UNKNOWN_GLYPH_WIDTH;
       metrics->underline_position = -PANGO_SCALE;
@@ -1650,7 +1851,7 @@ pango_font_get_metrics (PangoFont        *font,
 
 /**
  * pango_font_get_font_map:
- * @font: a #PangoFont, or %NULL
+ * @font: (nullable): a #PangoFont, or %NULL
  *
  * Gets the font map for which the font was created.
  *
@@ -1662,8 +1863,8 @@ pango_font_get_metrics (PangoFont        *font,
  * alive.  In most uses this is not an issue as a #PangoContext holds
  * a reference to the font map.
  *
- * Return value: (transfer none): the #PangoFontMap for the font, or %NULL
- *               if @font is %NULL.
+ * Return value: (transfer none) (nullable): the #PangoFontMap for the
+ *               font, or %NULL if @font is %NULL.
  *
  * Since: 1.10
  **/
@@ -1679,6 +1880,38 @@ pango_font_get_font_map (PangoFont *font)
     return NULL;
 }
 
+/**
+ * pango_font_get_hb_font: (skip)
+ * @font: a #PangoFont
+ *
+ * Get a hb_font_t object backing this font.
+ *
+ * Note that the objects returned by this function
+ * are cached and immutable. If you need to make
+ * changes to the hb_font_t, use hb_font_create_sub_font().
+ *
+ * Returns: (transfer none) (nullable): the hb_font_t object backing the
+ *          font, or %NULL if the font does not have one
+ *
+ * Since: 1.44
+ */
+hb_font_t *
+pango_font_get_hb_font (PangoFont *font)
+{
+  PangoFontPrivate *priv = pango_font_get_instance_private (font);
+
+  g_return_val_if_fail (PANGO_IS_FONT (font), NULL);
+
+  if (priv->hb_font)
+    return priv->hb_font;
+
+  priv->hb_font = PANGO_FONT_GET_CLASS (font)->create_hb_font (font);
+
+  hb_font_make_immutable (priv->hb_font);
+
+  return priv->hb_font;
+}
+
 G_DEFINE_BOXED_TYPE (PangoFontMetrics, pango_font_metrics,
                      pango_font_metrics_ref,
                      pango_font_metrics_unref);
@@ -1704,11 +1937,11 @@ pango_font_metrics_new (void)
 
 /**
  * pango_font_metrics_ref:
- * @metrics: a #PangoFontMetrics structure, may be %NULL
+ * @metrics: (nullable): a #PangoFontMetrics structure, may be %NULL
  *
  * Increase the reference count of a font metrics structure by one.
  *
- * Return value: @metrics
+ * Return value: (nullable): @metrics
  **/
 PangoFontMetrics *
 pango_font_metrics_ref (PangoFontMetrics *metrics)
@@ -1723,7 +1956,7 @@ pango_font_metrics_ref (PangoFontMetrics *metrics)
 
 /**
  * pango_font_metrics_unref:
- * @metrics: a #PangoFontMetrics structure, may be %NULL
+ * @metrics: (nullable): a #PangoFontMetrics structure, may be %NULL
  *
  * Decrease the reference count of a font metrics structure by one. If
  * the result is zero, frees the structure and any associated
@@ -1781,6 +2014,28 @@ pango_font_metrics_get_descent (PangoFontMetrics *metrics)
   return metrics->descent;
 }
 
+/**
+ * pango_font_metrics_get_height:
+ * @metrics: a #PangoFontMetrics structure
+ *
+ * Gets the line height from a font metrics structure. The
+ * line height is the distance between successive baselines
+ * in wrapped text.
+ *
+ * If the line height is not available, 0 is returned.
+ *
+ * Return value: the height, in Pango units
+ *
+ * Since: 1.44
+ */
+int
+pango_font_metrics_get_height (PangoFontMetrics *metrics)
+{
+  g_return_val_if_fail (metrics != NULL, 0);
+
+  return metrics->height;
+}
+
 /**
  * pango_font_metrics_get_approximate_char_width:
  * @metrics: a #PangoFontMetrics structure
@@ -1989,6 +2244,28 @@ pango_font_family_is_monospace (PangoFontFamily  *family)
     return FALSE;
 }
 
+/**
+ * pango_font_family_is_variable:
+ * @family: a #PangoFontFamily
+ *
+ * A variable font is a font which has axes that can be modified to
+ * produce different faces.
+ *
+ * Return value: %TRUE if the family is variable
+ *
+ * Since: 1.44
+ **/
+gboolean
+pango_font_family_is_variable (PangoFontFamily  *family)
+{
+  g_return_val_if_fail (PANGO_IS_FONT_FAMILY (family), FALSE);
+
+  if (PANGO_FONT_FAMILY_GET_CLASS (family)->is_variable)
+    return PANGO_FONT_FAMILY_GET_CLASS (family)->is_variable (family);
+  else
+    return FALSE;
+}
+
 /*
  * PangoFontFace
  */
@@ -2071,9 +2348,9 @@ pango_font_face_get_face_name (PangoFontFace *face)
 /**
  * pango_font_face_list_sizes:
  * @face: a #PangoFontFace.
- * @sizes: (out) (array length=n_sizes): location to store a pointer
- *         to an array of int. This array should be freed with
- *         g_free().
+ * @sizes: (out) (array length=n_sizes) (nullable) (optional):
+ *         location to store a pointer to an array of int. This array
+ *         should be freed with g_free().
  * @n_sizes: location to store the number of elements in @sizes
  *
  * List the available sizes for a font. This is only applicable to bitmap
@@ -2103,3 +2380,50 @@ pango_font_face_list_sizes (PangoFontFace  *face,
       *n_sizes = 0;
     }
 }
+
+/**
+ * pango_font_has_char:
+ * @font: a #PangoFont
+ * @wc: a Unicode character
+ *
+ * Returns whether the font provides a glyph for this character.
+ *
+ * Returns %TRUE if @font can render @wc
+ *
+ * Since: 1.44
+ */
+gboolean
+pango_font_has_char (PangoFont *font,
+                     gunichar   wc)
+{
+  PangoCoverage *coverage = pango_font_get_coverage (font, pango_language_get_default ());
+  PangoCoverageLevel result = pango_coverage_get (coverage, wc);
+  pango_coverage_unref (coverage);
+  return result != PANGO_COVERAGE_NONE;
+}
+
+/**
+ * pango_font_get_features:
+ * @font: a #PangoFont
+ * @features: (out caller-allocates) (array length=len): Array to features in
+ * @len: the length of @features
+ * @num_features: (inout): the number of used items in @features
+ *
+ * Obtain the OpenType features that are provided by the font.
+ * These are passed to the rendering system, together with features
+ * that have been explicitly set via attributes.
+ *
+ * Note that this does not include OpenType features which the
+ * rendering system enables by default.
+ *
+ * Since: 1.44
+ */
+void
+pango_font_get_features (PangoFont    *font,
+                         hb_feature_t *features,
+                         guint         len,
+                         guint        *num_features)
+{
+  if (PANGO_FONT_GET_CLASS (font)->get_features)
+    PANGO_FONT_GET_CLASS (font)->get_features (font, features, len, num_features);
+}
