@@ -53,12 +53,20 @@ struct _PangoColor
  * The #GObject type for #PangoColor.
  */
 #define PANGO_TYPE_COLOR pango_color_get_type ()
+PANGO_AVAILABLE_IN_ALL
 GType       pango_color_get_type (void) G_GNUC_CONST;
 
+PANGO_AVAILABLE_IN_ALL
+GType       pango_attribute_get_type    (void) G_GNUC_CONST;
+
+PANGO_AVAILABLE_IN_ALL
 PangoColor *pango_color_copy     (const PangoColor *src);
+PANGO_AVAILABLE_IN_ALL
 void        pango_color_free     (PangoColor       *color);
+PANGO_AVAILABLE_IN_ALL
 gboolean    pango_color_parse    (PangoColor       *color,
 				  const char       *spec);
+PANGO_AVAILABLE_IN_1_16
 gchar      *pango_color_to_string(const PangoColor *color);
 
 
@@ -139,6 +147,9 @@ typedef struct _PangoAttrIterator PangoAttrIterator;
  * @PANGO_ATTR_FONT_FEATURES: OpenType font features (#PangoAttrString). Since 1.38
  * @PANGO_ATTR_FOREGROUND_ALPHA: foreground alpha (#PangoAttrInt). Since 1.38
  * @PANGO_ATTR_BACKGROUND_ALPHA: background alpha (#PangoAttrInt). Since 1.38
+ * @PANGO_ATTR_ALLOW_BREAKS: whether breaks are allowed (#PangoAttrInt). Since 1.44
+ * @PANGO_ATTR_SHOW: how to render invisible characters (#PangoAttrInt). Since 1.44
+ * @PANGO_ATTR_INSERT_HYPHENS: whether to insert hyphens at intra-word line breaks (#PangoAttrInt). Since 1.44
  *
  * The #PangoAttrType
  * distinguishes between different types of attributes. Along with the
@@ -174,7 +185,10 @@ typedef enum
   PANGO_ATTR_GRAVITY_HINT,	/* PangoAttrInt */
   PANGO_ATTR_FONT_FEATURES,	/* PangoAttrString */
   PANGO_ATTR_FOREGROUND_ALPHA,	/* PangoAttrInt */
-  PANGO_ATTR_BACKGROUND_ALPHA	/* PangoAttrInt */
+  PANGO_ATTR_BACKGROUND_ALPHA,	/* PangoAttrInt */
+  PANGO_ATTR_ALLOW_BREAKS,	/* PangoAttrInt */
+  PANGO_ATTR_SHOW,		/* PangoAttrInt */
+  PANGO_ATTR_INSERT_HYPHENS,	/* PangoAttrInt */
 } PangoAttrType;
 
 /**
@@ -182,17 +196,17 @@ typedef enum
  * @PANGO_UNDERLINE_NONE: no underline should be drawn
  * @PANGO_UNDERLINE_SINGLE: a single underline should be drawn
  * @PANGO_UNDERLINE_DOUBLE: a double underline should be drawn
- * @PANGO_UNDERLINE_LOW: a single underline should be drawn at a position
- * beneath the ink extents of the text being
- * underlined. This should be used only for underlining
- * single characters, such as for keyboard
- * accelerators. %PANGO_UNDERLINE_SINGLE should
- * be used for extended portions of text.
+ * @PANGO_UNDERLINE_LOW: a single underline should be drawn at a
+ *     position beneath the ink extents of the text being
+ *     underlined. This should be used only for underlining
+ *     single characters, such as for keyboard accelerators.
+ *     %PANGO_UNDERLINE_SINGLE should be used for extended
+ *     portions of text.
  * @PANGO_UNDERLINE_ERROR: a wavy underline should be drawn below.
- * This underline is typically used to indicate
- * an error such as a possilble mispelling; in some
- * cases a contrasting color may automatically
- * be used. This type of underlining is available since Pango 1.4.
+ *     This underline is typically used to indicate an error such
+ *     as a possible mispelling; in some cases a contrasting color
+ *     may automatically be used. This type of underlining is
+ *     available since Pango 1.4.
  *
  * The #PangoUnderline enumeration is used to specify
  * whether text should be underlined, and if so, the type
@@ -429,97 +443,184 @@ struct _PangoAttrFontFeatures
   gchar *features;
 };
 
+PANGO_AVAILABLE_IN_ALL
 PangoAttrType         pango_attr_type_register (const gchar        *name);
+PANGO_AVAILABLE_IN_1_22
 const char *          pango_attr_type_get_name (PangoAttrType       type) G_GNUC_CONST;
 
+PANGO_AVAILABLE_IN_1_20
 void             pango_attribute_init        (PangoAttribute       *attr,
 					      const PangoAttrClass *klass);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute * pango_attribute_copy        (const PangoAttribute *attr);
+PANGO_AVAILABLE_IN_ALL
 void             pango_attribute_destroy     (PangoAttribute       *attr);
+PANGO_AVAILABLE_IN_ALL
 gboolean         pango_attribute_equal       (const PangoAttribute *attr1,
 					      const PangoAttribute *attr2) G_GNUC_PURE;
 
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_language_new      (PangoLanguage              *language);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_family_new        (const char                 *family);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_foreground_new    (guint16                     red,
 					      guint16                     green,
 					      guint16                     blue);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_background_new    (guint16                     red,
 					      guint16                     green,
 					      guint16                     blue);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_size_new          (int                         size);
+PANGO_AVAILABLE_IN_1_8
 PangoAttribute *pango_attr_size_new_absolute (int                         size);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_style_new         (PangoStyle                  style);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_weight_new        (PangoWeight                 weight);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_variant_new       (PangoVariant                variant);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_stretch_new       (PangoStretch                stretch);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_font_desc_new     (const PangoFontDescription *desc);
 
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_underline_new           (PangoUnderline underline);
+PANGO_AVAILABLE_IN_1_8
 PangoAttribute *pango_attr_underline_color_new     (guint16        red,
 						    guint16        green,
 						    guint16        blue);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_strikethrough_new       (gboolean       strikethrough);
+PANGO_AVAILABLE_IN_1_8
 PangoAttribute *pango_attr_strikethrough_color_new (guint16        red,
 						    guint16        green,
 						    guint16        blue);
 
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_rise_new          (int                         rise);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_scale_new         (double                      scale_factor);
+PANGO_AVAILABLE_IN_1_4
 PangoAttribute *pango_attr_fallback_new      (gboolean                    enable_fallback);
+PANGO_AVAILABLE_IN_1_6
 PangoAttribute *pango_attr_letter_spacing_new (int                        letter_spacing);
 
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *pango_attr_shape_new           (const PangoRectangle       *ink_rect,
 						const PangoRectangle       *logical_rect);
+PANGO_AVAILABLE_IN_1_8
 PangoAttribute *pango_attr_shape_new_with_data (const PangoRectangle       *ink_rect,
 						const PangoRectangle       *logical_rect,
 						gpointer                    data,
 						PangoAttrDataCopyFunc       copy_func,
 						GDestroyNotify              destroy_func);
 
+PANGO_AVAILABLE_IN_1_16
 PangoAttribute *pango_attr_gravity_new      (PangoGravity     gravity);
+PANGO_AVAILABLE_IN_1_16
 PangoAttribute *pango_attr_gravity_hint_new (PangoGravityHint hint);
+PANGO_AVAILABLE_IN_1_38
 PangoAttribute *pango_attr_font_features_new (const gchar *features);
+PANGO_AVAILABLE_IN_1_38
 PangoAttribute *pango_attr_foreground_alpha_new (guint16 alpha);
+PANGO_AVAILABLE_IN_1_38
 PangoAttribute *pango_attr_background_alpha_new (guint16 alpha);
+PANGO_AVAILABLE_IN_1_44
+PangoAttribute *pango_attr_allow_breaks_new     (gboolean allow_breaks);
+PANGO_AVAILABLE_IN_1_44
+PangoAttribute *pango_attr_insert_hyphens_new   (gboolean insert_hyphens);
 
+/**
+ * PangoShowFlags:
+ * @PANGO_SHOW_NONE: No special treatment for invisible characters
+ * @PANGO_SHOW_SPACES: Render spaces, tabs and newlines visibly
+ * @PANGO_SHOW_LINE_BREAKS: Render line breaks visibly
+ * @PANGO_SHOW_IGNORABLES: Render default-ignorable Unicode
+ *      characters visibly
+ *
+ * These flags affect how Pango treats characters that are normally
+ * not visible in the output.
+ */
+typedef enum {
+  PANGO_SHOW_NONE        = 0,
+  PANGO_SHOW_SPACES      = 1 << 0,
+  PANGO_SHOW_LINE_BREAKS = 1 << 1,
+  PANGO_SHOW_IGNORABLES  = 1 << 2
+} PangoShowFlags;
+
+PANGO_AVAILABLE_IN_1_44
+PangoAttribute *pango_attr_show_new              (PangoShowFlags flags);
+
+PANGO_AVAILABLE_IN_ALL
 GType              pango_attr_list_get_type      (void) G_GNUC_CONST;
+PANGO_AVAILABLE_IN_ALL
 PangoAttrList *    pango_attr_list_new           (void);
+PANGO_AVAILABLE_IN_1_10
 PangoAttrList *    pango_attr_list_ref           (PangoAttrList  *list);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_list_unref         (PangoAttrList  *list);
+PANGO_AVAILABLE_IN_ALL
 PangoAttrList *    pango_attr_list_copy          (PangoAttrList  *list);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_list_insert        (PangoAttrList  *list,
 						  PangoAttribute *attr);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_list_insert_before (PangoAttrList  *list,
 						  PangoAttribute *attr);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_list_change        (PangoAttrList  *list,
 						  PangoAttribute *attr);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_list_splice        (PangoAttrList  *list,
 						  PangoAttrList  *other,
 						  gint            pos,
 						  gint            len);
+PANGO_AVAILABLE_IN_1_44
+void               pango_attr_list_update        (PangoAttrList  *list,
+                                                  int             pos,
+                                                  int             remove,
+                                                  int             add);
 
+PANGO_AVAILABLE_IN_1_2
 PangoAttrList *pango_attr_list_filter (PangoAttrList       *list,
 				       PangoAttrFilterFunc  func,
 				       gpointer             data);
 
+PANGO_AVAILABLE_IN_1_44
+GSList        *pango_attr_list_get_attributes    (PangoAttrList *list);
+
+PANGO_AVAILABLE_IN_1_44
+GType              pango_attr_iterator_get_type  (void) G_GNUC_CONST;
+
+PANGO_AVAILABLE_IN_ALL
 PangoAttrIterator *pango_attr_list_get_iterator  (PangoAttrList  *list);
 
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_iterator_range    (PangoAttrIterator     *iterator,
 						 gint                  *start,
 						 gint                  *end);
+PANGO_AVAILABLE_IN_ALL
 gboolean           pango_attr_iterator_next     (PangoAttrIterator     *iterator);
+PANGO_AVAILABLE_IN_ALL
 PangoAttrIterator *pango_attr_iterator_copy     (PangoAttrIterator     *iterator);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_iterator_destroy  (PangoAttrIterator     *iterator);
+PANGO_AVAILABLE_IN_ALL
 PangoAttribute *   pango_attr_iterator_get      (PangoAttrIterator     *iterator,
 						 PangoAttrType          type);
+PANGO_AVAILABLE_IN_ALL
 void               pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 						 PangoFontDescription  *desc,
 						 PangoLanguage        **language,
 						 GSList               **extra_attrs);
+PANGO_AVAILABLE_IN_1_2
 GSList *          pango_attr_iterator_get_attrs (PangoAttrIterator     *iterator);
 
 
+PANGO_AVAILABLE_IN_ALL
 gboolean pango_parse_markup (const char                 *markup_text,
 			     int                         length,
 			     gunichar                    accel_marker,
@@ -528,7 +629,9 @@ gboolean pango_parse_markup (const char                 *markup_text,
 			     gunichar                   *accel_char,
 			     GError                    **error);
 
+PANGO_AVAILABLE_IN_1_32
 GMarkupParseContext * pango_markup_parser_new (gunichar               accel_marker);
+PANGO_AVAILABLE_IN_1_32
 gboolean              pango_markup_parser_finish (GMarkupParseContext   *context,
                                                   PangoAttrList        **attr_list,
                                                   char                 **text,
