@@ -20,7 +20,8 @@
  */
 
 #include "config.h"
-#include "pango-fontmap.h"
+#include "pango-fontmap-private.h"
+#include "pango-fontset-private.h"
 #include "pango-impl-utils.h"
 #include <stdlib.h>
 
@@ -82,8 +83,8 @@ pango_font_map_create_context (PangoFontMap *fontmap)
  *
  * Load the font in the fontmap that is the closest match for @desc.
  *
- * Returns: (transfer full): the newly allocated #PangoFont loaded,
- *          or %NULL if no font matched.
+ * Returns: (transfer full) (nullable): the newly allocated #PangoFont
+ *          loaded, or %NULL if no font matched.
  **/
 PangoFont *
 pango_font_map_load_font  (PangoFontMap               *fontmap,
@@ -124,8 +125,8 @@ pango_font_map_list_families (PangoFontMap      *fontmap,
  * Load a set of fonts in the fontmap that can be used to render
  * a font matching @desc.
  *
- * Returns: (transfer full): the newly allocated #PangoFontset
- *          loaded, or %NULL if no font matched.
+ * Returns: (transfer full) (nullable): the newly allocated
+ *          #PangoFontset loaded, or %NULL if no font matched.
  **/
 PangoFontset *
 pango_font_map_load_fontset (PangoFontMap                 *fontmap,
@@ -275,6 +276,7 @@ pango_font_map_real_load_fontset (PangoFontMap               *fontmap,
  *  or freed.
  *
  * Since: 1.4
+ * Deprecated: 1.38
  **/
 const char *
 pango_font_map_get_shape_engine_type (PangoFontMap *fontmap)
