@@ -24,6 +24,7 @@
 
 #include "pango-renderer.h"
 #include "pango-impl-utils.h"
+#include "pango-layout-private.h"
 
 #define N_RENDER_PARTS 4
 
@@ -53,6 +54,7 @@ struct _PangoRendererPrivate
 {
   PangoColor color[N_RENDER_PARTS];
   gboolean color_set[N_RENDER_PARTS];
+  guint16 alpha[N_RENDER_PARTS];
 
   PangoLayoutLine *line;
   LineState *line_state;
@@ -104,7 +106,8 @@ to_device (PangoMatrix *matrix,
     }
 }
 
-G_DEFINE_ABSTRACT_TYPE (PangoRenderer, pango_renderer, G_TYPE_OBJECT)
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (PangoRenderer, pango_renderer, G_TYPE_OBJECT,
+                                  G_ADD_PRIVATE (PangoRenderer))
 
 static void
 pango_renderer_class_init (PangoRendererClass *klass)
@@ -118,16 +121,12 @@ pango_renderer_class_init (PangoRendererClass *klass)
   klass->prepare_run = pango_renderer_default_prepare_run;
 
   gobject_class->finalize = pango_renderer_finalize;
-
-  g_type_class_add_private (gobject_class, sizeof (PangoRendererPrivate));
 }
 
 static void
 pango_renderer_init (PangoRenderer *renderer)
 {
-  renderer->priv = G_TYPE_INSTANCE_GET_PRIVATE (renderer,
-						PANGO_TYPE_RENDERER,
-						PangoRendererPrivate);
+  renderer->priv = pango_renderer_get_instance_private (renderer);
   renderer->matrix = NULL;
 }
 
@@ -161,7 +160,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
 			    int               x,
 			    int               y)
 {
-  PangoLayoutIter *iter;
+  PangoLayoutIter iter;
 
   g_return_if_fail (PANGO_IS_RENDERER (renderer));
   g_return_if_fail (PANGO_IS_LAYOUT (layout));
@@ -178,7 +177,7 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
 
   pango_renderer_activate (renderer);
 
-  iter = pango_layout_get_iter (layout);
+  _pango_layout_get_iter (layout, &iter);
 
   do
     {
@@ -186,19 +185,19 @@ pango_renderer_draw_layout (PangoRenderer    *renderer,
       PangoLayoutLine *line;
       int              baseline;
 
-      line = pango_layout_iter_get_line_readonly (iter);
+      line = pango_layout_iter_get_line_readonly (&iter);
 
-      pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
-      baseline = pango_layout_iter_get_baseline (iter);
+      pango_layout_iter_get_line_extents (&iter, NULL, &logical_rect);
+      baseline = pango_layout_iter_get_baseline (&iter);
 
       pango_renderer_draw_layout_line (renderer,
 				       line,
 				       x + logical_rect.x,
 				       y + baseline);
     }
-  while (pango_layout_iter_next_line (iter));
+  while (pango_layout_iter_next_line (&iter));
 
-  pango_layout_iter_free (iter);
+  _pango_layout_iter_destroy (&iter);
 
   pango_renderer_deactivate (renderer);
 }
@@ -700,7 +699,7 @@ pango_renderer_draw_glyph_item (PangoRenderer    *renderer,
 				int               x,
 				int               y)
 {
-  if (G_UNLIKELY (text))
+  if (!text)
     {
       pango_renderer_draw_glyphs (renderer,
 				  glyph_item->item->analysis.font,
@@ -1143,6 +1142,7 @@ pango_renderer_deactivate (PangoRenderer *renderer)
  * @color: (allow-none): the new color or %NULL to unset the current color
  *
  * Sets the color for part of the rendering.
+ * Also see pango_renderer_set_alpha().
  *
  * Since: 1.8
  **/
@@ -1181,9 +1181,9 @@ pango_renderer_set_color (PangoRenderer    *renderer,
  *
  * Gets the current rendering color for the specified part.
  *
- * Return value: the color for the specified part, or %NULL
- *  if it hasn't been set and should be inherited from the
- *  environment.
+ * Return value: (transfer none) (nullable): the color for the
+ *   specified part, or %NULL if it hasn't been set and should be
+ *   inherited from the environment.
  *
  * Since: 1.8
  **/
@@ -1200,6 +1200,59 @@ pango_renderer_get_color (PangoRenderer   *renderer,
     return NULL;
 }
 
+/**
+ * pango_renderer_set_alpha:
+ * @renderer: a #PangoRenderer
+ * @part: the part to set the alpha for
+ * @alpha: an alpha value between 1 and 65536, or 0 to unset the alpha
+ *
+ * Sets the alpha for part of the rendering.
+ * Note that the alpha may only be used if a color is
+ * specified for @part as well.
+ *
+ * Since: 1.38
+ */
+void
+pango_renderer_set_alpha (PangoRenderer   *renderer,
+                          PangoRenderPart  part,
+                          guint16          alpha)
+{
+  g_return_if_fail (PANGO_IS_RENDERER_FAST (renderer));
+  g_return_if_fail (IS_VALID_PART (part));
+
+  if ((!alpha && !renderer->priv->alpha[part]) ||
+      (alpha && renderer->priv->alpha[part] &&
+       renderer->priv->alpha[part] == alpha))
+    return;
+
+  pango_renderer_part_changed (renderer, part);
+
+  renderer->priv->alpha[part] = alpha;
+}
+
+/**
+ * pango_renderer_get_alpha:
+ * @renderer: a #PangoRenderer
+ * @part: the part to get the alpha for
+ *
+ * Gets the current alpha for the specified part.
+ *
+ * Return value: the alpha for the specified part,
+ *   or 0 if it hasn't been set and should be
+ *   inherited from the environment.
+ *
+ * Since: 1.38
+ */
+guint16
+pango_renderer_get_alpha (PangoRenderer   *renderer,
+                          PangoRenderPart  part)
+{
+  g_return_val_if_fail (PANGO_IS_RENDERER_FAST (renderer), 0);
+  g_return_val_if_fail (IS_VALID_PART (part), 0);
+
+  return renderer->priv->alpha[part];
+}
+
 /**
  * pango_renderer_part_changed:
  * @renderer: a #PangoRenderer
@@ -1261,6 +1314,8 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
   PangoColor *bg_color = NULL;
   PangoColor *underline_color = NULL;
   PangoColor *strikethrough_color = NULL;
+  guint16 fg_alpha = 0;
+  guint16 bg_alpha = 0;
   GSList *l;
 
   renderer->underline = PANGO_UNDERLINE_NONE;
@@ -1296,6 +1351,14 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
 	  strikethrough_color = &((PangoAttrColor *)attr)->color;
 	  break;
 
+	case PANGO_ATTR_FOREGROUND_ALPHA:
+          fg_alpha = ((PangoAttrInt *)attr)->value;
+	  break;
+
+	case PANGO_ATTR_BACKGROUND_ALPHA:
+          bg_alpha = ((PangoAttrInt *)attr)->value;
+	  break;
+
 	default:
 	  break;
 	}
@@ -1311,6 +1374,11 @@ pango_renderer_default_prepare_run (PangoRenderer  *renderer,
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_BACKGROUND, bg_color);
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_UNDERLINE, underline_color);
   pango_renderer_set_color (renderer, PANGO_RENDER_PART_STRIKETHROUGH, strikethrough_color);
+
+  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_FOREGROUND, fg_alpha);
+  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_BACKGROUND, bg_alpha);
+  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_UNDERLINE, fg_alpha);
+  pango_renderer_set_alpha (renderer, PANGO_RENDER_PART_STRIKETHROUGH, fg_alpha);
 }
 
 /**
@@ -1340,10 +1408,9 @@ pango_renderer_set_matrix (PangoRenderer     *renderer,
  * Gets the transformation matrix that will be applied when
  * rendering. See pango_renderer_set_matrix().
  *
- * Return value: the matrix, or %NULL if no matrix has been set
- *  (which is the same as the identity matrix). The returned
- *  matrix is owned by Pango and must not be modified or
- *  freed.
+ * Return value: (nullable): the matrix, or %NULL if no matrix has
+ *  been set (which is the same as the identity matrix). The returned
+ *  matrix is owned by Pango and must not be modified or freed.
  *
  * Since: 1.8
  **/
@@ -1361,13 +1428,13 @@ pango_renderer_get_matrix (PangoRenderer *renderer)
  *
  * Gets the layout currently being rendered using @renderer.
  * Calling this function only makes sense from inside a subclass's
- * methods, like in its draw_shape<!---->() for example.
+ * methods, like in its draw_shape vfunc, for example.
  *
  * The returned layout should not be modified while still being
  * rendered.
  *
- * Return value: (transfer none): the layout, or %NULL if no layout is being
- *  rendered using @renderer at this time.
+ * Return value: (transfer none) (nullable): the layout, or %NULL if
+ *  no layout is being rendered using @renderer at this time.
  *
  * Since: 1.20
  **/
@@ -1386,13 +1453,13 @@ pango_renderer_get_layout (PangoRenderer *renderer)
  *
  * Gets the layout line currently being rendered using @renderer.
  * Calling this function only makes sense from inside a subclass's
- * methods, like in its draw_shape<!---->() for example.
+ * methods, like in its draw_shape vfunc, for example.
  *
  * The returned layout line should not be modified while still being
  * rendered.
  *
- * Return value: (transfer none): the layout line, or %NULL if no
- *   layout line is being rendered using @renderer at this time.
+ * Return value: (transfer none) (nullable): the layout line, or %NULL
+ *   if no layout line is being rendered using @renderer at this time.
  *
  * Since: 1.20
  **/
